# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proxy_error.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import proxy_common_pb2 as proxy__common__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11proxy_error.proto\x12\x16\x64ingodb.pb.proxy.error\x1a\x12proxy_common.proto\"G\n\x05\x45rror\x12.\n\x07\x65rrcode\x18\x01 \x01(\x0e\x32\x1d.dingodb.pb.proxy.error.Errno\x12\x0e\n\x06\x65rrmsg\x18\x02 \x01(\t*\x1f\n\x05\x45rrno\x12\x06\n\x02OK\x10\x00\x12\x0e\n\tEINTERNAL\x10\xf4\x03\x42\x1b\n\x16io.dingodb.proxy.error\x88\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'proxy_error_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026io.dingodb.proxy.error\210\001\001'
  _globals['_ERRNO']._serialized_start=138
  _globals['_ERRNO']._serialized_end=169
  _globals['_ERROR']._serialized_start=65
  _globals['_ERROR']._serialized_end=136
# @@protoc_insertion_point(module_scope)
